@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## A real Try Catch Block Implementation Using Perl Filter - ~/lib/Nice/Try.pm
-## Version v1.0.2
+## Version v1.0.3
 ## Copyright(c) 2021 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2020/05/17
-## Modified 2021/05/19
+## Modified 2021/05/29
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -23,7 +23,7 @@ BEGIN
     use Scalar::Util ();
     use List::Util ();
     use Want ();
-    our $VERSION = 'v1.0.2';
+    our $VERSION = 'v1.0.3';
     our $ERROR;
     our( $CATCH, $DIED, $EXCEPTION, $FINALLY, $HAS_CATCH, @RETVAL, $SENTINEL, $TRY, $WANTARRAY );
 }
@@ -39,6 +39,7 @@ sub import
     $hash->{debug} = 0 if( !CORE::exists( $hash->{debug} ) );
     $hash->{no_filter} = 0 if( !CORE::exists( $hash->{no_filter} ) );
     $hash->{debug_code} = 0 if( !CORE::exists( $hash->{debug_code} ) );
+    $hash->{dont_want} = 0 if( !CORE::exists( $hash->{dont_want} ) );
     # We check if we are running under tie and if so we cannot use Want features, 
     # because they would trigger a segmentation fault.
     $hash->{is_tied} = 0;
@@ -218,6 +219,7 @@ sub _parse
 {
     my $self = shift( @_ );
     my $elem = shift( @_ );
+    no warnings 'uninitialized';
     if( !Scalar::Util::blessed( $elem ) || !$elem->isa( 'PPI::Node' ) )
     {
         return( $self->_error( "Element provided to parse is not a PPI::Node object" ) );
@@ -620,6 +622,11 @@ EOT
             $try_block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
             
             my $try_sub = <<EOT;
+CORE::local \$Nice::Try::THREADED;
+if( \$INC{'threads.pm'} )
+{
+    \$Nice::Try::THREADED = threads->tid;
+}
 CORE::local \$Nice::Try::WANT;
 CORE::local ( \$Nice::Try::EXCEPTION, \$Nice::Try::DIED, \@Nice::Try::RETVAL, \@Nice::Try::VOID );
 CORE::local \$Nice::Try::TRY = CORE::sub
@@ -630,7 +637,7 @@ CORE::local \$Nice::Try::TRY = CORE::sub
 __FINALLY_BLOCK__ CORE::local \$Nice::Try::HAS_CATCH = $has_catch_clause;
 CORE::local \$Nice::Try::WANTARRAY = CORE::wantarray;
 EOT
-            if( !$self->{is_tied} )
+            if( !$self->{is_tied} && !$self->{dont_want} )
             {
                 $try_sub .= <<EOT;
 CORE::local \$Nice::Try::NOOP = sub
@@ -638,7 +645,7 @@ CORE::local \$Nice::Try::NOOP = sub
     my \$ref = CORE::shift( \@_ );
     CORE::return(sub{ CORE::return( \$ref ) });
 };
-if( CORE::defined( \$Nice::Try::WANTARRAY ) )
+if( CORE::defined( \$Nice::Try::WANTARRAY ) && !\$Nice::Try::THREADED )
 {
     eval "\\\$Nice::Try::WANT = Want::want( 'LIST' )
             ? 'LIST'
@@ -1207,16 +1214,17 @@ sub _process_caller
 {
     my $self = shift( @_ );
     my $where = shift( @_ );
-	my $elem = shift( @_ ) || return( '' );
+    my $elem = shift( @_ ) || return( '' );
+    no warnings 'uninitialized';
     return( $elem ) if( !$elem->children );
     foreach my $e ( $elem->elements )
     {
         my $content = $e->content // '';
-        $self->_messagef( 6, "Checking element: [%d] class %s with %d children and value '%s'\n", $e->line_number, $e->class, ( $e->can('elements') ? scalar( $e->elements ) : 0 ), $content );
+        # $self->_messagef( 6, "Checking element: [%d] class %s with %d children and value '%s'\n", $e->line_number, $e->class, ( $e->can('elements') ? scalar( $e->elements ) : 0 ), $content );
         my $class = $e->class;
         if( $class eq 'PPI::Token::Word' && $content =~ /^(?:CORE\::)?(?:GLOBAL\::)?caller$/ )
         {
-            $self->_message( 4, "Found caller, replacing with ", 'Nice::Try::caller_' . $where );
+            # $self->_message( 4, "Found caller, replacing with ", 'Nice::Try::caller_' . $where );
             $e->set_content( 'Nice::Try::caller_' . $where );
         }
         
@@ -1233,7 +1241,8 @@ sub _process_caller
 sub _process_loop_breaks
 {
     my $self = shift( @_ );
-	my $elem = shift( @_ ) || return( '' );
+    my $elem = shift( @_ ) || return( '' );
+    no warnings 'uninitialized';
     return( $elem ) if( !$elem->children );
     $self->_messagef( 5, "Checking %d elements for '$elem'", scalar( $elem->elements ) );
     foreach my $e ( $elem->elements )
@@ -1319,132 +1328,133 @@ sub _process_loop_breaks
 sub _serialize 
 {
     my $self   = shift( @_ );
-	my $ppi    = shift( @_ ) || return( '' );
-	my @tokens = $ppi->tokens;
+    my $ppi    = shift( @_ ) || return( '' );
+    no warnings 'uninitialized';
+    my @tokens = $ppi->tokens;
 
-	# The here-doc content buffer
-	my $heredoc = '';
+    # The here-doc content buffer
+    my $heredoc = '';
 
-	# Start the main loop
-	my $output = '';
-	foreach my $i ( 0 .. $#tokens ) {
-		my $Token = $tokens[$i];
+    # Start the main loop
+    my $output = '';
+    foreach my $i ( 0 .. $#tokens ) {
+        my $Token = $tokens[$i];
 
-		# Handle normal tokens
-		unless ( $Token->isa('PPI::Token::HereDoc') ) {
-			my $content = $Token->content;
+        # Handle normal tokens
+        unless ( $Token->isa('PPI::Token::HereDoc') ) {
+            my $content = $Token->content;
 
-			# Handle the trivial cases
-			unless ( $heredoc ne '' and $content =~ /\n/ ) {
-				$output .= $content;
-				next;
-			}
+            # Handle the trivial cases
+            unless ( $heredoc ne '' and $content =~ /\n/ ) {
+                $output .= $content;
+                next;
+            }
 
-			# We have pending here-doc content that needs to be
-			# inserted just after the first newline in the content.
-			if ( $content eq "\n" ) {
-				# Shortcut the most common case for speed
-				$output .= $content . $heredoc;
-			} else {
-				# Slower and more general version
-				$content =~ s/\n/\n$heredoc/;
-				$output .= $content;
-			}
+            # We have pending here-doc content that needs to be
+            # inserted just after the first newline in the content.
+            if ( $content eq "\n" ) {
+                # Shortcut the most common case for speed
+                $output .= $content . $heredoc;
+            } else {
+                # Slower and more general version
+                $content =~ s/\n/\n$heredoc/;
+                $output .= $content;
+            }
 
-			$heredoc = '';
-			next;
-		}
+            $heredoc = '';
+            next;
+        }
 
-		# This token is a HereDoc.
-		# First, add the token content as normal, which in this
-		# case will definitely not contain a newline.
-		$output .= $Token->content;
+        # This token is a HereDoc.
+        # First, add the token content as normal, which in this
+        # case will definitely not contain a newline.
+        $output .= $Token->content;
 
-		# Now add all of the here-doc content to the heredoc buffer.
-		foreach my $line ( $Token->heredoc ) {
-			$heredoc .= $line;
-		}
+        # Now add all of the here-doc content to the heredoc buffer.
+        foreach my $line ( $Token->heredoc ) {
+            $heredoc .= $line;
+        }
 
-		if ( $Token->{_damaged} ) {
-			# Special Case:
-			# There are a couple of warning/bug situations
-			# that can occur when a HereDoc content was read in
-			# from the end of a file that we silently allow.
-			#
-			# When writing back out to the file we have to
-			# auto-repair these problems if we aren't going back
-			# on to the end of the file.
+        if ( $Token->{_damaged} ) {
+            # Special Case:
+            # There are a couple of warning/bug situations
+            # that can occur when a HereDoc content was read in
+            # from the end of a file that we silently allow.
+            #
+            # When writing back out to the file we have to
+            # auto-repair these problems if we aren't going back
+            # on to the end of the file.
 
-			# When calculating $last_line, ignore the final token if
-			# and only if it has a single newline at the end.
-			my $last_index = $#tokens;
-			if ( $tokens[$last_index]->{content} =~ /^[^\n]*\n$/ ) {
-				$last_index--;
-			}
+            # When calculating $last_line, ignore the final token if
+            # and only if it has a single newline at the end.
+            my $last_index = $#tokens;
+            if ( $tokens[$last_index]->{content} =~ /^[^\n]*\n$/ ) {
+                $last_index--;
+            }
 
-			# This is a two part test.
-			# First, are we on the last line of the
-			# content part of the file
-			my $last_line = List::Util::none {
-				$tokens[$_] and $tokens[$_]->{content} =~ /\n/
-				} (($i + 1) .. $last_index);
-			if ( ! defined $last_line ) {
-				# Handles the null list case
-				$last_line = 1;
-			}
+            # This is a two part test.
+            # First, are we on the last line of the
+            # content part of the file
+            my $last_line = List::Util::none {
+                $tokens[$_] and $tokens[$_]->{content} =~ /\n/
+                } (($i + 1) .. $last_index);
+            if ( ! defined $last_line ) {
+                # Handles the null list case
+                $last_line = 1;
+            }
 
-			# Secondly, are their any more here-docs after us,
-			# (with content or a terminator)
-			my $any_after = List::Util::any {
-				$tokens[$_]->isa('PPI::Token::HereDoc')
-				and (
-					scalar(@{$tokens[$_]->{_heredoc}})
-					or
-					defined $tokens[$_]->{_terminator_line}
-					)
-				} (($i + 1) .. $#tokens);
-			if ( ! defined $any_after ) {
-				# Handles the null list case
-				$any_after = '';
-			}
+            # Secondly, are their any more here-docs after us,
+            # (with content or a terminator)
+            my $any_after = List::Util::any {
+                $tokens[$_]->isa('PPI::Token::HereDoc')
+                and (
+                    scalar(@{$tokens[$_]->{_heredoc}})
+                    or
+                    defined $tokens[$_]->{_terminator_line}
+                    )
+                } (($i + 1) .. $#tokens);
+            if ( ! defined $any_after ) {
+                # Handles the null list case
+                $any_after = '';
+            }
 
-			# We don't need to repair the last here-doc on the
-			# last line. But we do need to repair anything else.
-			unless ( $last_line and ! $any_after ) {
-				# Add a terminating string if it didn't have one
-				unless ( defined $Token->{_terminator_line} ) {
-					$Token->{_terminator_line} = $Token->{_terminator};
-				}
+            # We don't need to repair the last here-doc on the
+            # last line. But we do need to repair anything else.
+            unless ( $last_line and ! $any_after ) {
+                # Add a terminating string if it didn't have one
+                unless ( defined $Token->{_terminator_line} ) {
+                    $Token->{_terminator_line} = $Token->{_terminator};
+                }
 
-				# Add a trailing newline to the terminating
-				# string if it didn't have one.
-				unless ( $Token->{_terminator_line} =~ /\n$/ ) {
-					$Token->{_terminator_line} .= "\n";
-				}
-			}
-		}
+                # Add a trailing newline to the terminating
+                # string if it didn't have one.
+                unless ( $Token->{_terminator_line} =~ /\n$/ ) {
+                    $Token->{_terminator_line} .= "\n";
+                }
+            }
+        }
 
-		# Now add the termination line to the heredoc buffer
-		if ( defined $Token->{_terminator_line} ) {
-			$heredoc .= $Token->{_terminator_line};
-		}
-	}
+        # Now add the termination line to the heredoc buffer
+        if ( defined $Token->{_terminator_line} ) {
+            $heredoc .= $Token->{_terminator_line};
+        }
+    }
 
-	# End of tokens
+    # End of tokens
 
-	if ( $heredoc ne '' ) {
-		# If the file doesn't end in a newline, we need to add one
-		# so that the here-doc content starts on the next line.
-		unless ( $output =~ /\n$/ ) {
-			$output .= "\n";
-		}
+    if ( $heredoc ne '' ) {
+        # If the file doesn't end in a newline, we need to add one
+        # so that the here-doc content starts on the next line.
+        unless ( $output =~ /\n$/ ) {
+            $output .= "\n";
+        }
 
-		# Now we add the remaining here-doc content
-		# to the end of the file.
-		$output .= $heredoc;
-	}
+        # Now we add the remaining here-doc content
+        # to the end of the file.
+        $output .= $heredoc;
+    }
 
-	$output;
+    $output;
 }
 
 
@@ -1640,7 +1650,7 @@ Also since version 1.0.0, L<Nice::Try> is context aware:
 
 =head1 VERSION
 
-    v1.0.2
+    v1.0.3
 
 =head1 DESCRIPTION
 
@@ -1807,7 +1817,7 @@ Would produce:
         }
     }
 
-WIll yield: C<Called from package main in file ./test.pl at line 10>
+Will yield: C<Called from package main in file ./test.pl at line 10>
 
 =back
 
@@ -1963,9 +1973,11 @@ But B<you cannot do>:
         # do some cleanup here
     }
 
+Also, the C<use feature 'try'> expression must be in the relevant block where you use C<try-catch>. You cannot just put it in your C<BEGIN> block at the beginning of your script. If you have 3 subroutines using C<try-catch>, you need to put C<use feature 'try'> in each of them. See L<perl documentation on lexical effect|https://perldoc.perl.org/feature#Lexical-effect> for more explanation on this.
+
 It is probably a matter of time until this is fully implemented in perl as a regular non-experimental feature.
 
-So, L<Nice::Try> is quite unique and fill the missing features, but because it is purely in perl and not an XS module, it is slower than XS module like L<Syntax::Keyword::Try>. I am not sure the difference would be that noticeable, since the parsing by L<PPI> is now done using an XS module, which makes things very fast.
+So, L<Nice::Try> is quite unique and fills the missing features, but because it is purely in perl and not an XS module, it is slower than XS module like L<Syntax::Keyword::Try>. I am not sure the difference would be that noticeable, since the parsing by L<PPI> is now done using an XS module, which makes things very fast.
 
 =head1 FINALLY
 
